@@ -12,7 +12,7 @@
  *
  * Also finds out IP of specified MAC
  *
- * $Id: arping.c,v 1.16 2000/07/02 11:05:56 marvin Exp $
+ * $Id: arping.c,v 1.18 2000/07/31 02:42:02 marvin Exp $
  */
 /*
  *  Copyright (C) 2000 Marvin (marvin@nss.nu)
@@ -37,7 +37,7 @@
 #include <net/if.h>
 #include <net/if_arp.h>
 
-const float version = 0.6;
+const float version = 0.7;
 
 struct ether_addr *mymac;
 u_char eth_xmas[ETH_ALEN] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
@@ -46,7 +46,6 @@ u_char eth_target[ETH_ALEN];
 
 u_int ip_xmas = 0xffffffff;
 
-
 pcap_t *pcap;
 struct bpf_program bpf_prog;
 struct in_addr net,mask;
@@ -76,14 +75,14 @@ void usage(int ret)
 
 void alasend(int i)
 {
-	numsent++;
-	if (numsent > maxcount) {
+	if (numsent >= maxcount) {
 		sigint(numrecvd ? 0 : 1);
 	}
+	numsent++;
 	if (searchmac) {
 		libnet_build_icmp_echo(ICMP_ECHO,      /* type */ 
 				       0,              /* code */ 
-				       4321,           /* id */ 
+				       (short)random(),           /* id */ 
 				       htons(numsent-1), /* seq */ 
 				       NULL,           /* pointer to payload */
 				       0,              /* payload length */ 
@@ -157,9 +156,12 @@ void handlepacket(const char *unused, struct pcap_pkthdr *h, u_char *packet)
 		hicmp = (struct icmphdr*)((char*)hip + sizeof(struct iphdr));
 		
 		if ((htons(hicmp->type) == ICMP_ECHOREPLY)
-			&& !memcmp(eth->h_source, eth_target, ETH_ALEN)
-			&& !memcmp(eth->h_dest, mymac->ether_addr_octet,
-				   ETH_ALEN)) {
+		    && (
+			    (!memcmp(eth->h_source, eth_target, ETH_ALEN)
+			     || !memcmp(eth_target, eth_xmas, ETH_ALEN))
+				    )
+			    && !memcmp(eth->h_dest, mymac->ether_addr_octet,
+				       ETH_ALEN)) {
 			u_char *cp = eth->h_source;
 			if (!rawoutput) {
 				printf("%d bytes from ", h->len);
